       COMMENT *

This file is used to generate boot DSP code for the 250 MHz fiber optic
	timing board using a DSP56303 as its main processor and the
	8 channel IR video processor

	*
	PAGE    132     ; Printronix page width - 132 columns

; Include the boot and header files so addressing is easy
	INCLUDE "timhdr.asm"
	INCLUDE	"timboot.asm"

	ORG	P:,P:

CC	EQU	IRREV4+TIMREV5+IRREV4		; Until voodoo is fixed

; Put number of words of application in P: for loading application from EEPROM
	DC	TIMBOOT_X_MEMORY-@LCV(L)-1

; Set software to IDLE mode
START_IDLE_CLOCKING
	MOVE	#IDLE,R0		; Exercise clocks when idling
	MOVE	R0,X:<IDL_ADR
	BSET	#IDLMODE,X:<STATUS	; Idle after readout
	JMP     <FINISH			; Need to send header and 'DON'

; Keep the array idling when not reading out
IDLE	DO      Y:<NSR,IDL1     	; Loop over number of pixels per line
	MOVE    #CONTINUOUS_RESET,R0 	; Serial transfer on pixel
	JSR     <CLOCK  		; Go to it
	MOVE	#COM_BUF,R3
	JSR	<GET_RCV		; Check for FO or SSI commands
	JCC	<NO_COM			; Continue IDLE if no commands received
	ENDDO
	JMP     <PRC_RCV		; Go process header and command
NO_COM	NOP
IDL1
	MOVE    #<SHIFT_ODD_ROW,R0	; Address of parallel clocking waveform
	JSR     <CLOCK
	JMP     <IDLE

;  *****************  Readout routine  *****************

; Overall loop - transfer and read NPR lines
RDARRAY	BSET	#ST_RDC,X:<STATUS 	; Set status to reading out
	MOVE	#$0c3004,A		; Reset FIFOs on video processors
	JSR	<WR_BIAS
	JSR	<PCI_READ_IMAGE		; Get the PCI board reading the image
;	JSET	#TST_IMG,X:STATUS,SYNTHETIC_IMAGE

	MOVE	#<FRAME_INIT,R0
	JSR     <CLOCK

	DO      Y:<NPR,LPR
	MOVE	#<SHIFT_ODD_ROW,R0
	JSR     <CLOCK
	
	MOVE	Y:<NSR,A
; !!!	LSR	A			; 2 channels per system
; !!!	LSR	#3,A			; 8 channels per system
; !!!	LSR	#4,A			; 16 channels per system
	LSR	#5,A			; 32 channels per system
	NOP
	DO	A1,LSR
	MOVE	#<PIXEL_READ,R0
	JSR     <CLOCK
	NOP
LSR	NOP
LPR

; Restore the controller to non-image data transfer and idling if necessary
RDA_END	JCLR	#IDLMODE,X:<STATUS,NO_IDL ; Don't idle after readout
	MOVE	#IDLE,R0
	MOVE	R0,X:<IDL_ADR
	JMP	<RDC_E
NO_IDL	MOVE	#<TST_RCV,R0
	MOVE	R0,X:<IDL_ADR
RDC_E	JSR	<WAIT_TO_FINISH_CLOCKING
	BCLR	#ST_RDC,X:<STATUS	; Set status to not reading out
        JMP     <START

; Not yet implemented
ABR_RDC
CONT_RD

; ******  Minclude many routines not directly needed for readout  *******
	INCLUDE "timIRmisc.asm"


TIMBOOT_X_MEMORY	EQU	@LCV(L)

;  ****************  Setup memory tables in X: space ********************

; Define the address in P: space where the table of constants begins

	IF	@SCP("DOWNLOAD","HOST")
	ORG     X:END_COMMAND_TABLE,X:END_COMMAND_TABLE
	ENDIF

	IF	@SCP("DOWNLOAD","ROM")
	ORG     X:END_COMMAND_TABLE,P:
	ENDIF

; Application commands
	DC	'PON',POWER_ON
	DC	'POF',POWER_OFF
	DC	'SBV',SET_BIAS_VOLTAGES
	DC	'IDL',START_IDLE_CLOCKING
	DC	'OSH',OPEN_SHUTTER
	DC	'CSH',CLOSE_SHUTTER
	DC	'RDC',RDARRAY    
	DC	'CLR',CLEAR    

; Exposure and readout control routines
	DC	'SET',SET_EXPOSURE_TIME
	DC	'RET',READ_EXPOSURE_TIME
	DC	'SEX',START_EXPOSURE
	DC	'PEX',PAUSE_EXPOSURE
	DC	'REX',RESUME_EXPOSURE
	DC	'AEX',ABORT_ALL
	DC	'ABR',ABR_RDC
	DC	'CRD',CONT_RD
	DC	'SRM',SELECT_READOUT_MODE

; Support routines
	DC	'SBN',SET_BIAS_NUMBER
	DC	'SMX',SET_MUX
	DC	'CSW',CLR_SWS
	DC	'RCC',READ_CONTROLLER_CONFIGURATION 

END_APPLICATON_COMMAND_TABLE	EQU	@LCV(L)

	IF	@SCP("DOWNLOAD","HOST")
NUM_COM			EQU	(@LCV(R)-COM_TBL_R)/2	; Number of boot + 
							;  application commands
EXPOSING		EQU	CHK_TIM			; Address if exposing
CONTINUE_READING	EQU	CONT_RD 		; Address if reading out
	ENDIF

	IF	@SCP("DOWNLOAD","ROM")
	ORG     Y:0,P:
	ENDIF

; Now let's go for the timing waveform tables
	IF	@SCP("DOWNLOAD","HOST")
        ORG     Y:0,Y:0
	ENDIF

GAIN	DC	END_APPLICATON_Y_MEMORY-@LCV(L)-1

NSR     DC      560   	 	; Number Serial Read, prescan + image + bias
NPR     DC      512     	; Number Parallel Read
NSCLR	DC      NF_CLR  	; Fast clearing of the pixels
NPCLR   DC      NS_CLR    	; Slow clearing, each line 
DUMMY	DC	0,0		; Binning parameters (reserved)
TST_DAT	DC	0		; Temporary definition for test images
CONFIG	DC	CC		; Controller configuration
TRM_ADR	DC	0		; Test RAM memory error address

; Include the waveform table for the designated IR array
	INCLUDE "WAVEFORM_FILE" ; Readout and clocking waveform file

END_APPLICATON_Y_MEMORY	EQU	@LCV(L)

; End of program
	END

